%==========================================================================
%   TP Multivariable : premiere partie
%   Auteurs:        DP, SR, YM
%   Modifications:  DP-20090928
%   Contact:        alain.bock@epfl.ch
%   Assistants:     AB, LM
%==========================================================================
%==========================================================================

clear all, close all, clc


%==========================================================================
%%   0)    On charge le modele non lineaire 
%    -------------------------------------
%       0.1) Declaration des variables avec la toolbox "symbolique"
%       0.2) Variable d'etat et equation de la dynamique
%==========================================================================

%%  0.1) Declaration des variables
% coordonnees et leurs derivees
syms xi theta real
syms xid thetad real
% la notation @d est utilisee pour la derivee temporelle de @.
 
% entree du systeme
syms u real
 
% Parametres physiques
syms Cf mr mp Jr Jp R d g real
% Cf  = coefficient de frottement
% mr = masse d'une roue
% mp = masse du pendule
% Jr = inertie de la roue en son centre de gravite
% Jp = inertie du pendule en son centre de gravite
% R  = rayon de la roue
% d  = longeur de la moitie du pendule
% g  = constante reliee a la gravite

%--------------------------------------
%%  0.2) Variable d'etat et equation de la dynamique
% Variable d'etats
x = [xi; theta; xid; thetad];

% Derive de la variabe d'etats xd=f(x,u)
modeleNL

% equation non lineaire de la dynamique
syms f 
f = xd;


%==========================================================================
%%   1)    Linearisation des equ. autour d'un point d'equilibre (x_b,...)
%    --------------------------------------------------------------------
%       1.1) Linearisation du systeme f a l'aide de la toolbox "symbolique"
%       1.2) Definition numerique du point de fonctionnement (x_b,u_b)
%       1.3) Evaluation des matrices A et B au point nominal
%       1.4) Sauvegarde du modele linearise dans un fichier *.mat
%==========================================================================
 
%%  1.1) Linearisation du systeme f
% Declaration des matrices du systeme linearise
syms A B;
 
% A = %-?-%
% B = %-?-%

%--------------------------------------
%%  1.2) Definition numerique du point de fonctionnement (x_b,u_b)
% x_b = %-?-%
% u_b = %-?-%

%--------------------------------------
%%  1.3) Evaluation des matrices A et B au point nominal
% xi     = x_b(1); %-!-%
% theta  = x_b(2); %-!-%
% xid    = x_b(3); %-!-%
% thetad = x_b(4); %-!-%
% u      = u_b;    %-!-%
 
% Chargement des parametres physiques
parametresPhysiques;
 
% Evaluation des matrices du systeme linearise au point nominal
% A = %-?-%
% B = %-?-%
% C = %-?-%
% D = %-?-%
 
% A ce stade, A et B sont des fonctions des parametres physiques et des
% valeurs nominales (matrices numeriques)
 

%==========================================================================
%%   2)    Simulation et comparaison  (non lineaire vs lineaire)
%    -----------------------------------------------------------
%       2.1) Definition des conditions initiales pour les 4 etats
%       2.2) Travail sur simulink: simulationLinearisation.mdl
%       2.3) Affichage de l'evolution 2D du systeme
%       2.4) Validation du modele linearise 
%==========================================================================

%%  2.1) Definition des conditions initiales pour les 4 etats
x0=conditionsInitiales; % a executer si les conditions initiales changent

%%  2.2) Travail sur simulink: simulationLinearisation.mdl
open_system('simulationLinearisation')
% sim('simulationLinearisation') %-!-%

%%  2.3) Affichage de l'evolution 2D du systeme
%-!!!-%
% fig=figure('Name','evolution des etats du modele non lineaire');
% affichage(etatsNL.time,etatsNL.signals.values',fig)
% 
% fig=figure('Name','evolution des etats du modele linearise');
% affichage(etatsLin.time,etatsLin.signals.values',fig)


%==========================================================================
%%   3)    Discretisation du systeme linearise continu
%    -------------------------------------------------
%       3.1) Echantillonnage
%       3.2) Calcul de la matrice Psi
%       3.3) Calcul de Phi et Gam
%       3.4) Representation continue avec 'ss'
%       3.5) Calcul a l'aide de 'c2d' et comparaison
%       3.6) Comparaison sur simulink: simulationDiscretisation.mdl
%       3.7) Sauvegarde du modele discretise dans un fichier.mat
%       3.8) Validation de la discretisation
%==========================================================================
 
% ordre du systeme
n = size(A,1); 
 
%--------------------------------------
%%  3.1) Echantillonnage
% periode d echantillonnage
% h = %-?-%
 
%--------------------------------------
%%  3.2) Calcul de la matrice Psi
% matrice Psi
%-???-%
%
%
%
%
 
%--------------------------------------
%%  3.3) Calcul de Phi et Gam
% Phi = %-?-%
% Gam = %-?-%

% NB: gamma est une fonction de Matlab, c'est pourquoi Gam est prefere
 
%--------------------------------------
%%  3.4) Representation continue avec 'ss'
% Mc = %-?-%
 
%--------------------------------------
%%  3.5) Calcul a l'aide de 'c2d' et comparaison
% Comparaison avec la solution matlab directe (avec la commande 'c2d')
% Md = %-?-%

% Comparaison
% Phi_erreur = %-?-%
% Gam_erreur = %-?-%
 
% Mise a jour des matrices Phi et Gam pour la suite (valeurs de 'c2d')
% Phi = Md.a; %-!-%
% Gam = Md.b; %-!-%

%%  3.6) Comparaison sur simulink: simulationDiscretisation.mdl
open_system('simulationDiscretisation')
% sim('simulationDiscretisation') %-!-%

%--------------------------------------
%%  3.7) Sauvegarde du modele linearise discretise dans un fichier *.mat
% save modeles Phi Gam C D h x_b u_b n; %-!-%